/**
 * Counts all of the numbers from 1 to n in a subthread. The main method
 * peaks at the counter while the thread is running to keep track of its
 * progress.
 *
 * @author Christian Ratliff
 */
public class CounterThread1 extends Thread {

  /** Indicates the progress of the count, the value -1 is used by default
   * to detect whether the subthread has yet started. */
  protected long count = -1;

  /** The highest number to count. */
  protected long countMax;

  /**
   * Assigns the maximum counting value.
   *
   * @param countTo The maximum number to count to before exiting.
   */
  public CounterThread1(long countTo) {
    super();
    this.countMax = countTo;
  }

  /**
   *  Returns the current number during the count.
   */
  public long getCount() {
    return this.count;
  }

  /**
   * Changes the current number during the count.
   *
   * @param cnt The new number in the count.
   */
  public void setCount(long cnt) {
    this.count = cnt;
  }

  /**
   * Performs the actual work of the subthread by counting from 1 to
   * the number provided at construction time.
   */
  public void run() {
    System.out.println("THREAD: starting");
    long cnt = 1;

    while (cnt++ < this.countMax) {
      this.setCount(cnt);
    }
    System.out.println("THREAD: ending");
  }

  /**
   * Tests the CounterThread class.
   *
   * @param argv The string arguments provided on the command line.
   */
  public static void main(String[] argv) {
    long cnt = 0;
    long max = Long.parseLong(argv[0]);
    CounterThread1 cntThread = new CounterThread1(max);

    cntThread.start();
    
    while (cntThread.isAlive()) {
      System.out.println("MAIN: "+cntThread.getCount());
      cnt++;
    }
    System.out.println("MAIN: ending at "+cntThread.getCount()+", picked "+cnt+" values.");
  }
}


/* run with -Xint (interpretted only) */
