import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Reads numbers from stdin and tests them for primality. It
 * uses a ThreadGroup to watch the prime thread for unhandled
 * exceptions.
 *
 * @author Christian Ratliff
 */
public class Controller implements ThreadExceptionListener {

  /** The cache where found prime numbers will be stored. */
  private PrimeCache cache;
  /** A ThreadGroup derivative which receives the exceptions
   * emitted from watched subthreads. */
  private ThreadGroupWatcher group;
  /** Our naughty sieve subthread. */
  private SieveThread sieve;

  /**
   *
   */
  public Controller() {
    cache = new PrimeCache();
    group = new ThreadGroupWatcher("SieveWatcher");
    group.addExceptionListener(this);
    sieve = new SieveThread(group, 100, cache);
    sieve.start();
  }

  /**
   * 
   * @param event The information about the thread which raised the
   *       exception and the exception itself.
   */
  public void exceptionEmitted(ThreadExceptionEvent event) {
    System.out.println("");
    System.out.println("ERROR:");
    System.out.println("\tunhandled exception thread="+event.getSource().toString());
    System.out.println("\texception="+event.getException().toString());
  }

  /**
   *
   */
  public void readPrimes() {
    /* Read numbers to test from the input stream. */
    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
    String line;
    
    try {
      System.out.print("Type a number: ");

      while ((line = reader.readLine()) != null && line.compareTo("QUIT") != 0) {
	Integer n = new Integer(line);
	
	if (cache.isPrime(n)) {
	  System.out.println("Number ["+n.toString()+"] is prime.");
	} else {
	  System.out.println("Number ["+n.toString()+"] is composite.");
	}

	System.out.print("Type a number: ");
      }
    } catch (NumberFormatException nfe) {
    } catch (IOException ioe) { }

    if (this.sieve.isAlive()) {
      System.out.println("Controller exiting while thread is still alive.");
    } else {
      System.out.println("Controller exiting.");
    }
  }

  /**
   * 
   * @param argv The string arguments from the command line.
   */
  public static void main(String[] argv) {
    Controller con = new Controller();

    con.readPrimes();
  }
}
