import java.lang.Integer;
import java.lang.Thread;

/**
 * Searches a given range for prime numbers, adding each prime
 * it finds to a prime numbers cache. The sieve algorithm
 * works by establishing an array from 1 to n
 *
 * @author Christian Ratliff
 */
public class SieveThread extends Thread {
  /** The knockout array for prime numbers from 1 to n */
  boolean[] primeArray;
  /** The cache where found prime numbers will be stored. */
  PrimeCache cache;

  /**
   * Initializes the sieve instance by allocating the prime
   * array to the size indicated by the max prime parameter.
   *
   * @param group The thread group instance which governs
   *       this thread.
   * @param maxPrime The upper bound on the range.
   * @param cache The class which caches the found prime
   *       numbers.
   */
  public SieveThread(ThreadGroupWatcher group, int maxPrime, PrimeCache cache) {
    super(group, "SieveThread");
    this.primeArray = new boolean[maxPrime];
    this.cache = cache;
  }

  /**
   * 
   */
  public void run() {
    /* Fill in the array as if every number was prime. */
    for (int n = 0; n < this.primeArray.length; n++) {
      this.primeArray[n] = true;
    }

    /* Now run down the array picking a candidate prime number
     * and using it to drop out larger composites. */
    for (int p = 2; p < this.primeArray.length; p++) {
      /* If this number is really prime, then use it. */
      if (this.primeArray[p] == true) {

	/* Add it to the list of prime numbers. */
	this.cache.addPrimeToCache(new Integer(p));
	try { Thread.sleep(50); } catch (InterruptedException ie) { }

	/* Run through the numbers from p to length-1. */
	for (int n = p+1; n < this.primeArray.length; n++) {
	  /* Is this a composite of p? */
	  if (n % p == 0) {
	    this.primeArray[n] = false;
	  }
	}
      }
    }
    throw new NullPointerException("This is a test");
  }
}
