
import java.lang.Thread;
import java.lang.Throwable;
import java.util.EventObject;

/**
 * Provides all of the information necessary about
 * an uncaught exception emitted by a thread. Instances
 * of this class are provided through the
 * ThreadExceptionListener interface.
 *
 * @author Christian Ratliff
 */
public class ThreadExceptionEvent extends EventObject {
  /** The exception raised by the thread. */
  Throwable uncaughtThrowable;

  /**
   * Creates an instance of this event, providing both
   * the thread which raised the exception, and the
   * exception itself.
   *
   * @param t The thread which raised the unhandled
   *       exception.
   * @param e The 
   */
  public ThreadExceptionEvent(Thread t, Throwable e) {
    super(t); /* source */
    this.uncaughtThrowable = e;
  }

  /**
   * Returns a the exception emitted by the subthread.
   * The type is 'Throwable', since it can include JVM
   * errors.
   *
   * @return The exception emitted by the subthread.
   */
  public Throwable getException() {
    return this.uncaughtThrowable;
  }

}
