import java.lang.ThreadGroup;
import java.lang.Throwable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * 
 *
 * @author Christian Ratliff
 */
public class ThreadGroupWatcher extends ThreadGroup {

  /** The list of listeners to be notified whenever
   * a uncaught exception is received. */
  public Vector listeners;

  public ThreadGroupWatcher(String name) {
    super(name);
    this.listeners = new Vector(3);
  }

  /**
   *
   * @param listener
   */
  public void addExceptionListener(ThreadExceptionListener listener) {
    this.listeners.add(listener);
  }

  /**
   * Called by the JVM whenever a thread in this thread group
   * emits an unhandled exception. The thread and exception is
   * passed along to all interested parties as an event.
   *
   * @param t The thread which emitted the exception to the JVM.
   * @param e The exception itself.
   */
  public void uncaughtException(Thread t, Throwable e) {
    Enumeration enum = this.listeners.elements();
    ThreadExceptionEvent event = new ThreadExceptionEvent(t, e);

    while (enum.hasMoreElements()) {
      ThreadExceptionListener listener =
	(ThreadExceptionListener)enum.nextElement();

      try {
	listener.exceptionEmitted(event);
      } catch (Exception ex) { }
    }
  }
}
